/*
 *
 * File: CMContentManager.cpp
 *
 * Description:
 * Contains WCDMContentManager class which provides the interface to 
 * CM for the CM Utilities. See CMContentManager.h for definition.
 *  An instance of this class normally represents a CM session and
 *  most methods must not be called prior to a successful call to 
 *  openConnection().
 *
 * Version:
 * 1.0.0 01/2001
 */

#include <malloc.h>
#include <memory.h>
#include <string.h>
#include <stdio.h>
#include <iostream.h>
#include <fcntl.h>

#include "WCDMContentManager.h" 


/*
 * Method: WCDMContentManager()
 *
 * Description:
 * Constructs an "empty" WCDMContentManager. openConnection() must
 *  be successful before the connection can be used.
 *
 * Parameters:
 *  (none)
 *
 * Return:
 *  (none)
 */
WCDMContentManager::WCDMContentManager()
{
   m_hSession = (ULONG)NULL;
   memset(&m_RC, 0, sizeof(RCSTRUCT));
}

/*
 * Method: ~WCDMContentManager()
 *
 * Description:
 * Deletes a WCDMContentManager, freeing resources. If an open
 *  connection exists, closes it as well.
 *
 * Parameters:
 *  (none)
 *
 * Return:
 *  (none)
 */
WCDMContentManager::~WCDMContentManager()
{
   // Delete the session      
   closeConnection();
}

/*
 * Method: ULONG getResult()
 *
 * Description:
 * Returns the primary return code from the last CM call. Note that
 *  if the connection is shared by multiple threads it is the caller's
 *  responsibility to ensure this is the appropriate return code.
 *
 * Parameters:
 *  (none)
 *
 * Return:
 *  ULONG   -  the return code.
 */
ULONG WCDMContentManager::getResult()
{
   return m_RC.ulRC;
}

/*
 * Method: ULONG getExternalResult()
 *
 * Description:
 * Returns the external return code from the last CM call. Note that
 *  if the connection is shared by multiple threads it is the caller's
 *  responsibility to ensure this is the appropriate return code.
 *
 * Parameters:
 *  (none)
 *
 * Return:
 *  ULONG   -  the external return code.
 */
ULONG WCDMContentManager::getExternalResult()
{
   return m_RC.ulExtRC;
}

/*
 * Method: ULONG getExternalReason()
 *
 * Description:
 * Returns the external reason code from the last CM call. Note that
 *  if the connection is shared by multiple threads it is the caller's
 *  responsibility to ensure this is the appropriate reason code.
 *
 * Parameters:
 *  (none)
 *
 * Return:
 *  ULONG   -  the external reason code.
 */
ULONG WCDMContentManager::getExternalReason()
{
   return m_RC.ulExtReason;
}

/*
 * Method: ULONG freeCMMemory(PVOID)
 *
 * Description:
 *  Frees memory allocated by CM.
 *  Requires an open connection to CM (see openConnection()).
 *
 * Parameters:
 *  PVOID pBuffer -  the buffer to free
 *
 * Return:
 * ULONG -  the result.
 */
ULONG WCDMContentManager::freeCMMemory(PVOID pBuffer)
{
   //ASSERT(m_hSession != NULL);

   RCSTRUCT rc;
   SimLibFree(m_hSession, pBuffer, &rc);
   if (rc.ulRC == SIM_RC_OK)
   {
      pBuffer = NULL;
   }
   return rc.ulRC;
}

/*
 * Method: ULONG openConnection(char*, char*, char*, char*)
 *
 * Description:
 *  Opens a connection to the desired CM server; if the user is not
 *  already connected and NewPassword is supplied, also changes the
 *  user's password.
 *
 * Parameters:
 *  char* ServerName  -  the server which should be connected
 *  char* UserID      -  user's Id on this server
 *  char* Password -  user's current password for the Id
 *  char* NewPassword -  new password for the id or "" to keep the old one
 *
 * Return:
 * ULONG -  SIM_RC_OK if successful, otherwise a CM primary return code.
 */
ULONG WCDMContentManager::openConnection(char* ServerName, char* UserID, char* Password, char* NewPassword)
{

   RCSTRUCT rc, rcLogon;   // return code structures
   ULONG result;         // 'apparent' result: always SIM_RC_OK if a connection is made
   /***
   char* pszDBName       = new char[strlen(ServerName)+1];
   char* pszUserID       = new char[strlen(UserID)+1];
   char* pszPassword     = new char[strlen(Password)+1];
   char* pszNewPassword  = (char*)NULL;
   ***/
   UCHAR* pszDBName       = (UCHAR*)alloca(strlen(ServerName)+1);
   UCHAR* pszUserID       = (UCHAR*)alloca(strlen(UserID)+1);
   UCHAR* pszPassword     = (UCHAR*)alloca(strlen(Password)+1);
   UCHAR* pszNewPassword  = NULL;
   BITS   fSession       = SIM_SS_CONFIG;

   // In CM 7.1, we can try for unified logon, but not yet...
   //BITS   fSession       = UserID.IsEmpty() ? SIM_SS_NORMAL | SIM_SS_UNIFIED_LOGON : SIM_SS_NORMAL;

   strcpy((char*)pszDBName, ServerName);
   strcpy((char*)pszUserID, UserID);
   strcpy((char*)pszPassword, Password);

   //printf("[WCDMContentManager::openConnection] args=[%s, %s, %s]\n\n", pszDBName, pszUserID, pszPassword);

   //if (strlen(NewPassword) > 0)
   if (NewPassword != NULL && strlen(NewPassword) > 0)
   {
      pszNewPassword = (UCHAR*)alloca(strlen(NewPassword)+1);  
      strcpy((char*)pszNewPassword, NewPassword);
   }

   // Logon to server
   //SimLibLogon((UCHAR*)pszDBName, NULL, (UCHAR*)pszUserID, (UCHAR*)pszPassword, (UCHAR*)pszNewPassword, NULL, NULL, fSession, NULL, &rcLogon);
   SimLibLogon(pszDBName, NULL, pszUserID, pszPassword, pszNewPassword, NULL, NULL, fSession, NULL, &rcLogon);

   m_RC = rcLogon;      // save error info
   result = rcLogon.ulRC;

   //printf("[WCDMContentManager::openConnection] result=%i\n\n", result);

   if ( (result == SIM_RC_OK) ||
       (result == SIM_RC_GRACE_PERIOD_ENDED) ||
       (result == SIM_RC_GRACE_PERIOD_OVER_LIMIT) ||
       (result == SIM_RC_GRACE_PERIOD_UNDER_LIMIT) )
   {
      // Save session informatmion
      m_hSession  = (HSESSION)rcLogon.ulParam1;
      m_pLogonInfo = (USERLOGONINFOSTRUCT*) rcLogon.ulParam2;

      result = SIM_RC_OK;  // force a good result regardless of what else happened
   }

   if (pszNewPassword != NULL) 
   {
   }

   return result;
}

/*
 * Method: ULONG closeConnection()
 *
 * Description:
 *  Closes the connection with the CM server, if one exists.
 *
 * Parameters:
 *  (none)
 *
 * Return:
 * ULONG -  SIM_RC_OK if no connection exists or if the connection is successfully
 *          closed, otherwise a CM primary return code.
 */
ULONG WCDMContentManager::closeConnection()
{
   RCSTRUCT rc;
   rc.ulRC = SIM_RC_OK;
   if (m_hSession != NULL) 
   {
      // Free user info & logoff...
      freeCMMemory(m_pLogonInfo);
      SimLibLogoff(m_hSession, NULL, &rc);
      m_RC = rc;
   }

   return rc.ulRC;
}

/*
 * Method: BOOL isConnected()
 *
 * Description:
 *  Returns whether an open connection with CM exists.
 *
 * Parameters:
 *  (none)
 *
 * Return:
 * BOOL  -  TRUE if a connection exists, otherwise FALSE.
 */
BOOL WCDMContentManager::isConnected()
{
   return (m_hSession != NULL);
}

/*
 * Method: ULONG createAttribute()
 *
 * Description:
 *  Creates an attribute in the current system, using the language of the current user.
 *  Requires an open connection to CM (see openConnection()).
 *
 * Parameters:
 *  char* pszName        -  the name of the attribute
 *  USHORT usAttrType    -  the attribute type; must be one of the following:
 *                       SIM_ATTR_DATE, SIM_ATTR_DECIMAL, SIM_ATTR_FSTRING, SIM_ATTR_LONG,
 *                       SIM_ATTR_SH0RT, SIM_ATTR_TIME, SIM_ATTR_TIMESTAMP, SIM_ATTR_VSTRING
 *  LONG lMin            -  for STRING types, minimum length of the string; for LONG and SHORT,
 *                       the minimum value; for DECIMAL the number of digits to the right
 *                       of the decimal point
 *  LONG lMax            -  for STRING types, maximum length of the string; for LONG and SHORT,
 *                       the maximum value; for DECIMAL the maximum total number of digits
 *  BITS fTypeFlags      -  additional type information for STRINGs; must be one of the following:
 *                       SIM_ATTR_ALPHA, SIM_ATTR_ALPHANUM, SIM_ATTR_ALPHANUM_EXT,
 *                       SIM_ATTR_NONE, SIM_ATTR_NUMERIC, SIM_ATTR_OTHER
 *
 * Return:
 * ULONG -  SIM_RC_OK if the attribute is created successfully.
 */
ULONG WCDMContentManager::createAttribute(char* pszName, USHORT usAttrType, LONG lMin, LONG lMax, BITS fTypeFlags)
{
   //ASSERT(m_hSession != NULL);
   
   RCSTRUCT rc;
   
   ATTRINFOSTRUCT AttrInfo;
   memset(&AttrInfo, 0, sizeof(ATTRINFOSTRUCT));
   
   AttrInfo.ulStruct = sizeof(ATTRINFOSTRUCT);
   AttrInfo.fUseBidirectional = FALSE;
   AttrInfo.fSymmetricSwapping = FALSE;
   AttrInfo.fShaping = FALSE;

   AttrInfo.lMin = lMin;
   AttrInfo.lMax = lMax;
   AttrInfo.fTypeFlags = fTypeFlags;
   AttrInfo.usAttrType = usAttrType;

   AttrInfo.usHorizontalOrientation = SIM_ATTR_LEFT_TO_RIGHT;
   AttrInfo.usMode = SIM_ATTR_IMPLICIT;
   AttrInfo.usNumericSelectionDefault = SIM_ATTR_ARABIC;
   
   strncpy(AttrInfo.szAttributeName, pszName, SIM_ATTR_NAME_LENGTH);
   AttrInfo.szAttributeName[SIM_ATTR_NAME_LENGTH] = '\0';
   strncpy(AttrInfo.achLanguageCode, m_pLogonInfo->szUserLanguage, SIM_LANGUAGE_CODE_LENGTH);

   Ip2CreateAttr(m_hSession, &AttrInfo, NULL, &rc);

   m_RC = rc;
   return rc.ulRC;
}

